@@ -4,11 +4,11 @@ use Moo;
 
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
-our $VERSION = '0.000006';
+our $VERSION = '0.000007';
 
 use Class::Inspector ();
 use Class::Unload    ();
-use Data::Printer;
+use Data::Dumper qw( Dumper );
 use List::Util qw( any );
 use Log::Dispatch::Array ();
 use Module::Runtime qw( require_module );
@@ -232,7 +232,7 @@ sub _list_to_hash {
     my $pkg  = shift;
     my $list = shift;
 
-    $self->logger->debug( 'list to hash: ' . $pkg . "\n" . np($list) );
+    $self->logger->debug( 'list to hash: ' . $pkg . "\n" . Dumper($list) );
 
     my %hash;
     for my $item ( @{$list} ) {
@@ -365,7 +365,7 @@ EOF
         $logger->log(
             level   => $level,
             message => sprintf(
-                "Problem trying to eval %s:\n%s",
+                "Problem trying to eval %s\n%s",
                 $pkg,
                 $msg,
             ),
@@ -448,7 +448,7 @@ App::perlimports::ExportInspector - Inspect code for exportable symbols
 
 =head1 VERSION
 
-version 0.000006
+version 0.000007
 
 =head1 SYNOPSIS
 
